# -*- coding: utf-8 -*-
from inferal import *
import copy


def seer_infer(t, alive, now_day):  # 占い先から占い師の推理をする
    table = copy.deepcopy(t)  # dict汚染を防ぐ為の仮想テーブルを作成する
    clone_table = copy.deepcopy(t)  # 投票推理の仮想テーブルを作成する

    villager_base = ave_villager_wolf_pattern(table, now_day, alive)  # デフォルト村人の狼率
    clone_table = vote_convert(clone_table, now_day)  # 投票推理テーブルに情報をインプット
    for v in role_player_id(table, u"SEER"):
        for r in table[v][u"result"]:
            target = r[1]
            result_wolf = -1 if r[2] == u"WEREWOLF" else 1
            change_seer_infer = villager_base - cal(clone_table[target][u"infer_role"])  # 高ければ高い程標準値より人外寄り

            # 人外率はデフォルトより高いほど○時の真占い度が減るが、●時の真占い度は増える
            table[v][u"infer_role"] = alt(table[v][u"infer_role"], [[1.0 + change_seer_infer * result_wolf, u"HUMAN"]])

    return table


def i_am_true(t, my):  # 自分の評価を100%村人にする
    table = copy.deepcopy(t)
    table[my[u'id']][u"infer_role"][u"WEREWOLF"] = 0.0
    table[my[u'id']][u"infer_role"][u"POSSESSED"] = 0.0
    return table


def role_die_set(t, alive): # 占いが噛まれた場合、対抗占いの内訳を修正する
    table = copy.deepcopy(t)  # dict汚染を防ぐ

    seer = role_player_id(table, u"SEER")  # 占いのIDを取得する
    medium = role_player_id(table, u"MEDIUM")  # 霊能のIDを取得する
    seer_pattern = BB.seer_pattern(len(seer), len(medium))  # 占いの内訳を取得する
    die_seer = [v for v in seer if v not in alive]
    alive_seer = [v for v in seer if v in alive]
    if len(die_seer) == 0 or len(alive_seer) == 0:
        return table

    # print("ROLE_DIE_SET START")
    alt_default = {u"WEREWOLF": 0.0,u"HUMAN": 0.0,u"POSSESSED": 0.0}

    # 死んだプレイヤーのinferを合計する
    for die in die_seer:
        for v in table[die][u"infer_role"].items():
            alt_default[v[0]] += v[1]

    # 新たなalt用テーブルを作成する
    for v in range(0, len(seer_pattern) - 1):
        role = seer_pattern[v][1]
        seer_pattern[v][0] =- alt_default[role] / len(alive_seer)
        # 基本的には噛まれたプレイヤー分の狼率を分配する感じ
        # 吊られて霊○のプレイヤーの分も分配する

    for v in alive_seer:  # 生存しているプレイヤー全てに補正をかける
        table[v][u"infer_role"] = alt(table[v][u"infer_role"], seer_pattern)

    return table


def infer(table, alive, today, my):  # 推理をする
    t = copy.deepcopy(table)  # 仮想テーブルを作る

    t = i_am_true(t, my)  # 自分を真100%にする
    t = combine_pattern(t, alive, today)  # 全ての役職のデータを内訳に合わせる
    t = role_die_set(t, alive)  # 占いの内訳を死んだ人の結果に合わせる
    t = medium_convert(t, today)  # 霊能結果をコンバートする
    t = seer_error_check(t, today)  # 占いの破綻チェックをする
    t = seer_infer(t, alive, today)  # 占い先から占い師の推理をする
    t = seer_result_convert(t)  # 占いの占い先の影響を与える
    t = vote_convert(t, today)  # 投票先推理をする

    return t


# 投票処理を一纏めで行う
def create_vote(table, alive, today, my):
    t = infer(table, alive, today, my)  # 色々な推理を一纏めにして行う
    inf = make_infer_table(t, alive, today)  # 推理テーブルを作成する
    """ 身内にはなるべく投票しない　この機能を切る
    if my[u'role'] == u'WEREWOLF':
        inf = fix_infer_wolf_family(inf, my[u'family'], alive)
    """
    return int(vote_target(t, alive, inf, my[u'id'], today, my))


# 推理テーブルを作成する
def create_infer(table, alive, today, my):
    t = infer(table, alive, today, my)  # 色々な推理を一纏めにして行う
    return make_infer_table(t, alive, today)  # 推理テーブルを作成する


# 狼テーブルを作成する
def create_wolf_table(table, alive, today, my):
    t = infer(table, alive, today, my)  # 色々な推理を一纏めにして行う
    inf = make_wolf_table(t, alive)  # 狼テーブルを作成する
    return get_wolf_list(inf,  my[u'id'], my)  # 狼リストを返す