# -*- coding: utf-8 -*-
# 推理システムを利用したアルゴリズム（投票先の分析とか）
from inference import BigBrother
from algorithm import *
import copy
import random

BB = BigBrother()
"""
table:拡張テーブル
action:行動ロール
"""

# --- テーブル非編集処理 ---

# 信用に値する今日の霊能結果があるか否かをチェックし、返す
# 値する情報を入手=霊能結果を返す
# 値する情報を入手出来ない=霊能結果を返さない
# 霊能結果:(day, target, result)
def medium_check(table, day):  # 拡張テーブル、取得したい日時

    def no_dup_check(med_list):  # 霊能結果リストを渡す。全ての霊能が同じ結果を出していれば、その値を信用する
        result = med_list[0]  # とりあえず1番最初に所得するデータ
        for v_dup in med_list:
            if result != v_dup:
                return None
        return result

    # 霊能一覧の取得
    get = role_player(table, "MEDIUM")
    if len(get) == 0:
        v = None
    elif len(get) == 1:
        v = role_result(get[0][u"result"], day)  # 素直に結果を返す
    else:  # 複数霊能の今日の値が重複していれば信用する
        v = no_dup_check([role_result(k[u"result"], day) for v,k in table.items()])
    return v


def medium_list(table, now_day):  # 信用に値する霊能結果のリストを手に入れる Noneの場合は消す　空リストが返却される可能性有
    # table 拡張テーブル
    # day 今の日時
    medium = [medium_check(table,d) for d in range(2, now_day+1)]
    return except_none(medium)


def wolf_sum(table, now_day):  # 霊能結果からの狼の合計を求める
    return sum([1 for v in medium_list(table, now_day) if v[2] == u"WEREWOLF"])


def ave_villager_wolf_pattern(table, now_day, alive):  # 村人率の計算

    seer = role_player_id(table, u"SEER")  # 占いのIDを取得する
    medium = role_player_id(table, u"MEDIUM")  # 霊能のIDを取得する
    seer_pattern = BB.seer_pattern(len(seer), len(medium))  # 占いの内訳を取得する

    sum_villager_wolf = 3 - wolf_sum(table, now_day) - seer_pattern[2][0] * len(role_player_id(table, u"SEER"))  # グレーの狼数の合計
    if sum_villager_wolf < int(len(alive) / 2):  # 最高人外数を上回ってたら修正する
        sum_villager_wolf == int(len(alive) / 2)
    return sum_villager_wolf / len(alive)  # 人外率


# --- テーブル編集処理 ---
# 基本的に仮想テーブルにのみ使用した方がいいだろう

def medium_convert(t, now_day):  # 霊能結果をコンバートする  day, target, result
    table = copy.deepcopy(t)
    medium = medium_list(table, now_day)
    if len(medium) == 0:
        return table
    for v in medium:  # 霊能結果を取り出す
        if v[2] == u"HUMAN":  # 村人判定
            c = [[0.0, u"WEREWOLF"]]
        else:  # 人狼判定
            c = [[0.0, u"HUMAN"]]
        table[v[1]][u"infer_role"] = alt(table[v[1]][u"infer_role"], c)

    return table


def vote_convert(t, now_day):  # 投票結果を使って色々計算する
    # 修正 霊●プレイヤー→プレイヤー全員
    # 長所 精度が上がる
    # 短所 処理が重くなる

    # table 拡張テーブル
    # day 今の日にち
    table = copy.deepcopy(t)
    clone_table = copy.deepcopy(table)

    player_list = [v for v in table]
    [BB.vote_infer_wolf_per(clone_table, v, cal_w(table[v][u"infer_role"])) for v in player_list]

    #medium_wolf = [v for v in medium_list(table,now_day) if v[2] == u"WEREWOLF"]  # 霊結果●のプレイヤーを狼修正する
    #[BB.vote_infer_wolf_per(table, v[1], 1.0) for v in medium_wolf]  # 霊結果●のプレイヤーの投票先などを村目にする

    return clone_table
    # 全てのプレイヤーを考慮する場合
    # [BB.vote_infer_wolf_per(table, v[1], 1.0) for v in table]


def seer_error_check(t, today):  # 占い師の破綻をチェックする
    # 霊●を占っていないか、占い●吊り→霊●になってるかのチェック
    # 詰み計算は他に出来る人が居ないのでパス

    # 占い●を出したプレイヤー一覧を取得
    # それの霊能情報を取得。None→変動なし　●→真寄り（非狂狼目）　○→人外目（非真目）
    table = copy.deepcopy(t)
    medium = medium_list(table, today)
    if len(medium) == 0:
        return table

    medium_id = [v[1] for v in medium]  # 霊能プレイヤーリストを作成

    for v in role_player_id(table, "SEER"):
        for r in table[v][u"result"]:  # 結果を取得する
            if r[1] in medium_id:  # 占い先が霊能結果（処刑されたプレイヤーリスト）に入っていたら
                medium_result = medium[medium_id.index(r[1])]  # 霊能結果を取得する
                if r[2] != medium_result[2]:  # 霊能結果と占い結果が違っていた場合
                    table[v][u"infer_role"][u"HUMAN"] = 0.0  # その占い師を人外判定する

    # 吊られたプレイヤーを取得
    return table


def combine_pattern(t, alive, now_day):  # 全ての役職のデータを標準値に合わせる
    table = copy.deepcopy(t)  # dict汚染を防ぐ

    seer = role_player_id(table, u"SEER")  # 占いのIDを取得する
    medium = role_player_id(table, u"MEDIUM")  # 霊能のIDを取得する
    seer_pattern = BB.seer_pattern(len(seer), len(medium))  # 占いの内訳を取得する
    medium_pattern = BB.medium_pattern(len(seer), len(medium))  # 霊能の内訳を取得する

    # 村人の処理
    # TypeError: 'NoneType' object has no attribute '__getitem__'
    sum_villager_wolf = 3 - wolf_sum(table, now_day) - seer_pattern[2][0] * len(seer)  # グレーの狼数の合計
    if sum_villager_wolf < int(len(alive)/2):  # 最高人外数を上回ってたら修正する
        sum_villager_wolf == int(len(alive)/2)
    villager_wolf_percent = sum_villager_wolf / len(alive)  # 人外率
    villager_pattern = [[1.0 - villager_wolf_percent, u"HUMAN"], [villager_wolf_percent, u"WEREWOLF"]]

    for v in seer:  # 占いの値を標準化する
        table[v][u"infer_role"] = alt(table[v][u"infer_role"], seer_pattern)

    for v in medium:  # 霊能の値を標準化する
        table[v][u"infer_role"] = alt(table[v][u"infer_role"], medium_pattern)

    for v in role_player_id(table, u"VILLAGER"):  # 村人の値を標準化する
        table[v][u"infer_role"] = alt(table[v][u"infer_role"], villager_pattern)

    return table


# --- 推理テーグル箇所 ---
# --- テーブル非編集処理 ---

# 推理テーブルを作る
# 低ければ低い程使い勝手が良い
# 人外値を返す　高ければ高い程人外寄りになる
def make_infer_table(table, alive, day):

    infer_table = {v:cal(table[v][u"infer_role"]) for v in alive}  # プレイヤーリストを作る

    """
    for v in role_player_id(table, u"SEER"):  # 占いは人外率75%になるまで基本的には処刑しない
        if cal(table[v][u"infer_role"]) > 0.75:
            infer_table[v] = 1.0
        else:
            infer_table[v] = 0.0
    """
    seer_list = role_player_id(table, u"SEER")
    for v in seer_list:  # 占いは人外率が高いがなるべく処刑しない
        if v in alive:
            if day <= 5-len(seer_list):  # （5-占い数）日目までは占いは処刑しない
                infer_table[v] = 0.0
            else:
                infer_table[v] /= 2.3

    medium = role_player_id(table, u"MEDIUM")  # 霊能は2人以上居たらなるべく殺す
    for v in medium:
        if v in alive:
            if len(medium) == 1:
                infer_table[v] = 0.0
            else:
                infer_table[v] = 0.3

    return infer_table

"""
infer_tableの構造
{
    AgentID:処刑優先度（ベース）
}
"""


def make_wolf_table(table, alive):
    return {v:cal_w(table[v][u"infer_role"]) for v in alive}  # プレイヤーリストを作る


def get_wolf_list(infer_table, not_id,  my):  # 狼候補
    if not_id is None:
        not_id = []

    vote_target_result = sorted(infer_table.items(), key=lambda x: x[1])
    vote_target_result.reverse()
    if my[u'role'] == u"POSSESSED":
        vote_target_result.reverse()

    wolf_list = []

    for v in vote_target_result:
        if v[0] not in not_id:
            wolf_list.append(v[0])
        if len(wolf_list) == 3:
            return wolf_list

    return wolf_list


def vote_target(table, alive, infer_table, not_id, now_day, my):  # 投票先を決める

    # table 拡張テーブル
    # alive 生存者テーブル
    # not_id プレイヤーリストに入れないID
    # listはデフォルト引数に出来ない為仕方がない
    if not_id is None:
        not_id = []


    # 予告されてる場合は(1.0/生存者数)^予告者数で計算する

    # 投票先補正は（思ったよりうちのエージェントの能力が高かったしあまり効果がないので）オミット
    """
    td_coefficient = 1.15 if now_day <= 4 else 1.0  # 投票予告受けてるプレイヤーへの投票補正値
    # 4日目（短期人狼換算で5日目）まで便乗票を採用する　根拠は感覚

    # 予告投票されているプレイヤーリストを取得する
    vote_list = [table[v][u"today_vote"] for v in table if table[v][u"today_vote"] is not None]
    for v in vote_list:  # 投票を確認したらカウントする
        if v in alive:
            infer_table[v] *= td_coefficient
    """

    # 入手したデータ（dict）をtupleの逆順ソートする
    vote_target_result = sorted(infer_table.items(), key=lambda x: x[1])
    vote_target_result.reverse()
    if my[u'role'] == u"POSSESSED":
        vote_target_result.reverse()

    # 非処刑リストに入ってないかチェックをする
    for v in vote_target_result:
        if v[0] not in not_id:
            return v[0]

    return 1  # バグ回避に適当に返す


def seer_target(t, a, infer_table, my_id):  # 占い候補を決める
    # 現在の傾向:入手したリストから人外率の高いプレイヤーを返す
    # table 拡張テーブル
    # alive 生存者情報
    # my_id 自分のID
    table = copy.deepcopy(t)
    alive = copy.deepcopy(a)

    human_list = except_role(table, alive)  # 生存者から役職を除外する

    # 占われてない村人を取得する。（プレイヤーリストから各自の占い候補を除外する形になる）
    no_divined_player = except_table_seer_target(role_player(table, u"SEER"), human_list)

    if len(no_divined_player) == 0:  # もし空だったら自分の占い先のみを除外する
        no_divined_player = except_table_seer_target([table[my_id]], human_list)
    if len(no_divined_player) == 0:  # それでも空なら生きてる人間を適当に返す
        return alive[0]

    target_list = infertable_sort({v: infer_table[v] for v in infer_table if v in no_divined_player}.items())
    target_list.reverse()

    return target_list[0][0]


def kill_target(table, alive, my):  # 噛み候補を決める
    # 最も値（噛む度）の高いプレイヤーを襲撃する
    # 仮組みなので後々変更が必要になる
    # table 拡張テーブル
    # alive 生存者情報
    # bite_data 噛みデータ (日にち、対象, 可否)

    if my[u'family']:
        alive_wolf = my[u'family']
    else:
        alive_wolf = [my[u'id']]  # 生存狼リストを作る

    player_list = {v: 0.0 for v in alive if v not in alive_wolf}  # 狼を除外したプレイヤーリストを作る

    # 1人だけの霊能が居たら噛み評価値を最高（1.0）にする
    medium = role_player_id(table, "MEDIUM")
    if len(medium) == 1 and medium[0] in alive:
        player_list[medium[0]] = 1.0

    seer = role_player_id(table, "SEER")
    if len(seer) == 1 and seer[0] in alive:
        player_list[seer[0]] = 1.0

    elif len(seer) == 2: # 占い2なら先にどっちかは噛んでおく
        for v in seer:
            if v in alive:
                player_list[v] = 0.8

    elif len(seer) == 3:  # 占い3ならあまり噛まないようにする
        for v in seer:
            if v in alive:
                player_list[v] = 0.2

    # 生存している狼を投票していたら+0.3
    for wolf in alive_wolf:
        for v in player_list:
            value = sum([1 for vote in table[v][u'vote'] if vote[1] in alive_wolf])
            player_list[v] += 0.6 * value

    target_list = infertable_sort(player_list.items())
    return target_list[0][0]


def seer_result_convert(table):  # 占い結果から導く情報をtableに返す
    seer = role_player_id(table, "SEER")
    for v in seer:
        BB.seer_target_convert(table, table[v])

    return table


def guard_target(table, alive):  # 護衛候補を決める
    # 最も値（噛む度）の高いプレイヤーを護衛する
    # table 拡張テーブル
    # alive 生存者情報
    player_list = {v: 0.0 for v in alive}  # 狼を除外したプレイヤーリストを作る

    # 1人だけの霊能が居たら噛み評価値を最高（1.0）にする
    medium = role_player_id(table, u"MEDIUM")
    if len(medium) == 1 and medium[0] in alive:
        player_list[medium[0]] = 1.0

    seer = role_player_id(table, u"SEER")
    if len(seer) == 1 and seer[0] in alive:
        player_list[seer[0]] = 1.0

    elif len(seer) == 2:  # 占い2なら先にどっちかは噛んでおく
        for v in seer:
            if v in alive:
                player_list[v] = 0.9

    elif len(seer) == 3:  # 占い3ならあまり噛まないようにする
        for v in seer:
            if v in alive:
                player_list[v] = 0.2

    target_list = infertable_sort(player_list.items())
    return target_list[0][0]


def change_infer_table(table):  # 推理テーブルを変更する
    pass


def make_seer_target(my, table, alive, day, infer, plan):  # 占い結果を作る
    role = my[u'role']
    my_id = my[u'id']
    if role == u"POSSESSED":

        if plan == "KILL_MEDIUM_AND_KILL_SEER":
            # 1人だけの霊能が生きててしかも●出してないなら●判定
            medium = role_player_id(table, u"MEDIUM")
            if len(medium) == 1:
                target_medium = medium[0]
                if target_medium in alive and target_medium not in [v[1] for v in table[my_id][u"result"]]:
                    return int(target_medium),  u'WEREWOLF'

            seer = role_player_id(table, u"SEER")

            # 2人だけの占いが居て（自分含め）しかも●出してないなら対抗に●判定
            if len(seer) == 2:
                seer.remove(my_id)
                target_seer = seer[0]
                if target_seer in alive and target_seer not in [v[1] for v in table[my_id][u"result"]]:
                    return int(target_seer),  u'WEREWOLF'

        return int(seer_target(table, alive, infer, my_id)), u"HUMAN"

    elif role == u"WEREWOLF":

        if plan == "1DAY_CUT_FAMILY":

            if day == 1:
                # 相方に●出し
                target_list = [v for v in my[u"family"] if v != my_id]
                return int(target_list[0]), u"WEREWOLF"

            elif day == 2:
                first_target = table[my_id][u'result'][0][1]
                target_list = [v for v in my[u"family"] if v != my_id and v != first_target]
                return int(target_list[0]), u"HUMAN"



        return int(seer_target(table, alive, infer, my_id)), u"HUMAN"

    pass


def fix_infer_wolf_family(i, family, alive):
    infer = copy.deepcopy(i)
    for v in family:
        if v in alive:
            infer[v] *= 0.5  # 相方の狼率は0.5倍にする

    return infer