# -*- coding: utf-8 -*-
#import pandas as pd
#import numpy as np
from algorithm import *


# 本来はクラスを使う必要性が無いが、将来的にロジスティック回帰を組み込めるようにわざと冗長な設計にしている
# 本当なら自然言語処理用クラス等も欲しい所
class BigBrother:

    def __init__(self):
        # 本来ならpandasの初期定義とか行う所だが、そもそも機械学習が今の仕様だと使えないので無事オミット

        # 推理基本テーブル
        # alt(infer, self.infer["VOTE"]["WEREWOLF"], )
        self.infer = {
            "VOTE": {  # 投票された場合の狼値変動
                "WEREWOLF": [[0.7, "WEREWOLF"]],
                "POSSESSED": [[0.95, "WEREWOLF"]],

                "DAY1_0VOTE": [[0.9, "HUMAN"]],  # 初日0投票
            },

            "DIVINED": {  # 占い判定
                "WOLF_KILL": {  # 狼判定を処刑した時
                    "WEREWOLF": [[0.1, "WEREWOLF"],[0.3, "POSSESSED"]],# 狼判定
                    "HUMAN": [  [0.0, "HUMAN"],]# 村判定
                },

                # 占い師の役職による○の信頼度の変動
                "SEER_ROLE": {
                    u"HUMAN": {
                        "WEREWOLF": [[0.9, "HUMAN"]],
                        "POSSESSED": [[0.95, "HUMAN"]],
                        "HUMAN": [[0.0, "WEREWOLF"]]
                    },
                    u"WEREWOLF": {
                        "WEREWOLF": [[0.0, "WEREWOLF"]],
                        "POSSESSED": [[0.8, "WEREWOLF"]],
                        "HUMAN": [[0.0, "HUMAN"]]
                    }
                },
            },

            "MEDIUM": {  # 霊能関連処理
                "HUMAN": [[0.0, "WEREWOLF"]],  # 霊能村判定
                "WEREWOLF": [[0.0, "HUMAN"], [0.0, "POSSESSED"]]  # 霊能狼判定
            },
            "BITE": {
                "DIE": [[0.0, "WEREWOLF"], [0.8, "POSSESSED"]],  # 噛まれたプレイヤーはやや真よりにする
                "VOTE": [[0.8, "HUMAN"]]  # 噛まれたプレイヤーの投票先は狼寄りに修正
            },
            "OTHER": {  # その他（メタなど）
                # "DAY1_0VOTE": [0.9, "HUMAN"],  # 初日0投票
            }
        }

    # dayStart時に動作させるもの（霊能結果に依存しない推理）
    def vote_infer_dayStart(self, table, day_vote, alive):  # 投票先から朝一推理する
        # table いつもの拡張テーブル
        # day_vote 今日の投票結果（game_info出力）[{"agent":7,"day":1,"target":10}]
        # alive 生存プレイヤーリスト

        # 0票のプレイヤーを全員狼寄りにする
        vote_table = {v: 0 for v in alive}  # プレイヤーリストを作る
        for v in day_vote:  # 投票を確認したらカウントする
            if v[u"target"] == 0:
                continue
            vote_table[str(v[u"target"])] += 1  # KeyError: '0'

        no_vote_player = [v for v in vote_table if vote_table[v] == 0]  # 0票貰いのプレイヤーの名前を入手する

        for target in no_vote_player:  # それらのプレイヤーの村率を下げる
            table[target][u"infer_role"] = alt(table[target][u"infer_role"], self.infer["VOTE"]["DAY1_0VOTE"])
        #return table

    def bite_infer_dayStart(self, table, bite_player_id):  # 噛まれた人の投票先を狼目に修正
        # table 拡張テーブル
        # bite_player_id 噛まれたプレイヤーのID

        # 噛まれたプレイヤーの狼度を0に、狂人度を若干減らす
        table[str(bite_player_id)][u"infer_role"] = alt(table[str(bite_player_id)][u"infer_role"], self.infer["BITE"]["DIE"])

        for v in table[str(bite_player_id)][u"vote"]:  # 投票を取り出す
            target = v[1]
            if target == '0':
                continue
            table[target][u"infer_role"] = alt(table[target][u"infer_role"], self.infer["BITE"]["VOTE"])  # 投票先のプレイヤーを狼寄りにする

    def vote_infer_wolf_per(self, table, target_address, wolf_score):  # 狼投票者・被投票者の村率を上げる
        # table 拡張推理テーブル
        # target_address チェック対象者のID
        # wolf_score=狼スコア:狼の確率（0.0~1.0）の間で計算をする

        # ターゲット投票者の狼率を下げる
        for v,k in table.items():
            get = sum([1 for vote in k[u"vote"] if vote[1] == target_address])  # 対象ターゲットに投票した回数を取得する
            if get:
                # table[v][u'infer_role'][u'HUMAN'] *= self.vote_wolf * get * wolf_score
                table[v][u'infer_role'] = alt(table[v][u'infer_role'], self.infer["VOTE"]["WEREWOLF"] * get, wolf_score)

        # ターゲット被投票者の狼率を下げる
        target_vote = table[target_address][u"vote"]
        for v in target_vote:
            target = v[1]  # ターゲット投票者の投票先(day, target)
            # table[t][u'infer_role'][u'HUMAN'] *= self.vote_wolf * wolf_score
            table[target][u'infer_role'] = alt(table[target][u'infer_role'], self.infer["VOTE"]["WEREWOLF"], wolf_score)

    def seer_pattern(self, s, m):  # 占い師の役職パターンを占い数と霊能数から取得する
        # 占い全員殺すパターン
        # 占い4以上, 3-2, 占い数+霊能数>=6（人外全露出） 占い+霊能>=5（全役職ロラで残り1W）&占い!=1

        if s == 0:  # 占い師が0～1人の時は霊能の数に左右されない
            return [[0.0, u"HUMAN"], [0.0, u"POSSESSED"], [0.0, u"WEREWOLF"]]  # バグ回避
        if s == 1:
            return [[1.0, u"HUMAN"], [0.0, u"POSSESSED"], [0.0, u"WEREWOLF"]]  # 真確

        if m <= 1:  # 霊能が1人か居ない場合
            if s == 2:  # 2-1
                return [[0.5,u"HUMAN"],[0.5,u"POSSESSED"], [0.0,u"WEREWOLF"]]  # 真狂=1:1
            elif s == 3:  # 3-1
                return [[0.33, u"HUMAN"], [0.33, u"POSSESSED" ], [0.34, u"WEREWOLF"]]  # 真狂狼=1:1:1
        elif m == 2:  # 霊能が二人の場合
            if s == 2:  # 2-2
                return [[0.5, u"HUMAN"], [0.2, u"POSSESSED"], [0.3, u"WEREWOLF"]]  # 真狂狼=5:2:3

        return [[0.0, u"HUMAN"], [1.0, u"POSSESSED"], [0.0, u"WEREWOLF"]]  # 全員殺す

    def medium_pattern(self, s, m):  # 霊能のパターンを占いと霊能の数から取得する（将来的に機械学習で変動させられるようにしたい）
        if m == 0:
            return [[0.0, u"HUMAN"], [0.0, u"POSSESSED"], [0.0, u"WEREWOLF"]]  # バグ回避
        if m == 1:
            return [[1.0,u"HUMAN"], [0.0,u"POSSESSED" ], [0.0,u"WEREWOLF" ]]  # 真確

        return [[0.0,u"HUMAN"], [1.0,u"POSSESSED"], [0.0,u"WEREWOLF"]]  # 霊能3以上なら全員殺す

    def seer_target_convert(self, table, chara):  # 占い師の占い結果から導いた結果をtableに挿入する
        # chara 占い師（単体）
        # table 推理テーブル（単体）
        for p in chara[u"result"]:  # 占い結果をコンバートする
            c = p[1]  # 占い対象ID
            r = p[2]  # 占い結果
            # 占い師の役職と○の傾向を照らし合わせ挿入する
            for v,k in chara[u"infer_role"].items():  # 拡張テーブルの値から計算する.
                table[c][u'infer_role'] = alt(table[c][u'infer_role'], self.infer["DIVINED"]["SEER_ROLE"][r][v], k)
