# -*- coding: utf-8 -*-
from aiwolfpy.player import BasePlayer
import aiwolfpy.templatetalkfactory as ttf
import aiwolfpy.templatewhisperfactory as twf
from aiwolfpy.data import RoleList, SpeciesList
from algorithm import *
from inferal import *
from inference import BigBrother
from big_infer import *
from pprint import pprint


BB = BigBrother()
Role = RoleList()
Species = SpeciesList()
# clientstarter.py -h localhost -p 10000

# 役職ごとの戦術リスト
# 将来的に、いろんな人が戦術を作り、それらを相手の手に応じて変えていくスタイルが取れれば最高だと思います
TACTICS = {
    u'VILLAGER':u'ALL_STEALTH',  # 村人戦術は完ステ
    u'SEER': u'NORMAL',  # 占い戦術は普通のCO
    u'MEDIUM' : u'NORMAL',  # 普通のCO
    u'WEREWOLF' : u'1DAY_CUT_FAMILY', # u'1DAY_CUT_FAMILY', "NORMAL"  # 初手身内切り  # 将来的には、狼の場合は戦術のタイプなどを選択可能にする予定
    u'POSSESSED' : u'KILL_MEDIUM_AND_KILL_SEER',  # 霊能●→占い●作戦
}

# 発言パターンは初日処刑率ワーストの候補を流用する
TALK_PATTERN = ["VOTE", "ESTIMATE WOLF", "ESTIMATE MEDIUM", "ESTIMATE SEER", "RESET_VOTE"]

class SimplePlayer(BasePlayer):

    def __init__(self, game_info, game_setting):
        player_list = game_info[u'statusMap'].keys()  # プレイヤーのリスト
        self.my = {
            u'id':str(game_info[u'agent']),  # int
            u'role':game_info[u'roleMap'][str(game_info[u'agent'])],
        }
        if self.my[u'role'] == u'WEREWOLF':
            self.my[u'family'] = [value for value in game_info[u'roleMap']]  # 狼情報
            self.my[u'hoax'] = u"VILLAGER"# u"SEER" # 騙り役職。今回は身内切り固定なので占い固定

        if self.my[u'role'] == u'POSSESSED':
            self.my[u'hoax'] = u"SEER"

        self.player_table = {
            value:{  # エージェントごとの拡張情報。直書きしてるのはPythonの仕様
                u"alive": True,  # 生存情報 使ってない
                u"state": "Alive",  # 現在の情報 Alive（生存） Vote（投票死）　Bite（処刑死）
                u"assamed_role": u"VILLAGER",  # 表面上の役職
                # u"true_role": None,  # 本当の役職
                u"result": [],  # 役職の結果など(day, target, result)
                u"infer_role": {
                    # 役職ごとの確率 村人の場合は双方100%で計算して、それぞれを減算する
                    # 潜伏占い/霊能/狂人の可能性は完全に切る。計算した所で有意義に活かせる機会なんてない
                    u"WEREWOLF": 1.0,
                    u"HUMAN": 1.0,
                    u"POSSESSED": 0.0  # 処理用
            },
            u"vote": [],  # 投票結果はこちらに保存する  (day, target) (int, str)
            u"today_vote": None, # 今日の投票先を保存する (target) (str)
            u"target_value":0.0, # 処刑優先評価値 未使用
            u"cco": False,  # CCOか否か
        } for value in player_list}

        self.alive_player = [value for value, value2 in self.player_table.items() if value2[u'alive'] == True]  # 生存中のプレイヤーリスト

        # 役職処理に使うもの
        self.today = 0  # 日時の取得
        self.co = False  # 自分がCOしたかどうか
        self.role_results = ()  # 役職の結果データ

        self.bite = []  # 噛みテーブル(day, target, result) 例:(3, u'14', True/False)
        self.vote_table = {}  # 合計投票結果の保存先
        self.agentIdx = game_info['agent']

        # 発言処理に使うものリスト
        self.count = 0
        self.vote_target = 0  # 投票先ターゲット
        self.wolf_estimate_list = []
        self.wolf_estimate_count = 0
        self.talk_set = True


    def dayStart(self, game_info):
        # talkに利用するデータを初期化
        self.count = 0
        self.vote_target = 0  # 投票先ターゲット
        self.wolf_estimate_list = []
        self.wolf_estimate_count = 0
        self.talk_set = True

        # 使用するデータを初期化
        self.today = game_info[u'day'] # 日時データ

        # 初期化
        # 投票結果をテーブルに挿入する
        vote_data = game_info[u'voteList']  # 投票結果を入れる
        if len(vote_data) != 0:
            [self.player_table[str(v[u"agent"])][u"vote"].append((self.today - 1, str(v[u"target"]))) for v in
             vote_data]

        # 投票・噛み推理をする
        # 現在実装中

        # 投票推理
        if len(game_info[u'voteList']) != 0:
            BB.vote_infer_dayStart(self.player_table, game_info[u'voteList'], self.alive_player)  # 役職などの結果に依存しない推理をする

        # 噛み推理
        if game_info[u"attackedAgent"] != -1:
            BB.bite_infer_dayStart(self.player_table, game_info[u"attackedAgent"])

        # エージェントの生存情報を更新・今日の投票先を初期化
        # 推理の後に書く理由:処理が楽だから

        self.alive_player = [value for value, key in game_info[u'statusMap'].items() if key == u'ALIVE']  # 生存中のプレイヤーリストをゲットする
        for v, k in self.player_table.items():
            self.player_table[v][u"alive"] = True if v in self.alive_player else False  # 生存情報を挿入する
            self.player_table[v][u"today_vote"] = None  # 今日の投票先を初期化する

        # 占いと霊能の結果を取得/作成する
        role = self.my[u'role']
        if role == u"SEER":
            # 占い文章テンプレ {u'target': 1, u'day': 1, u'agent': 5, u'result': u'HUMAN'}
            r = game_info[u'divineResult']
            if r is not None:
                self.role_results = (r[u"target"], r[u"result"])  # 占い結果の格納  int,str

        elif role == u'MEDIUM':
            r = game_info[u'mediumResult']
            if r is not None:
                self.role_results = (r[u"target"], r[u"result"])  # 占い結果の格納  int,str

        elif role == u'WEREWOLF':
            if self.my[u'hoax'] == u"SEER":  # 占い結果の作成
                t = infer(self.player_table, self.alive_player, self.today, self.my)  # 色々な推理を一纏めにして行う
                inf = make_infer_table(t, self.alive_player, self.today)  # 推理テーブルを作成する
                # 占い結果を作成する
                self.role_results = make_seer_target(self.my, self.player_table, self.alive_player, self.today, inf, TACTICS[u'WEREWOLF'])

        elif role == u'POSSESSED':
            t = infer(self.player_table, self.alive_player, self.today, self.my)  # 色々な推理を一纏めにして行う
            inf = make_infer_table(t, self.alive_player, self.today)  # 推理テーブルを作成する
            # 占い結果を作成する
            self.role_results = make_seer_target(self.my, self.player_table, self.alive_player, self.today, inf, TACTICS[u'POSSESSED'])

    def talk(self, talk_history, whisper_history):
        # 役職・投票先検知
        self.player_table = detection(self.player_table ,talk_history)  # 発言の取得

        # 発言の内容
        # 役職ごとに発言を変える
        role = self.my[u'role']
        if role == u"WEREWOLF" or role == u"POSSESSED":  # 狼の場合は設定した騙り役職に合わせる
            role = self.my[u'hoax']

        if role == u"SEER":
            # 0日目は削除されるので新しいバージョンで試す場合は消す
            if self.today == 0:
                if not self.co:
                    self.co = True
                    return ttf.comingout(int(self.my[u'id']), "SEER")  # 占いCOをする

            elif self.role_results:
                # 表示用の占い結果を格納する
                r = self.role_results
                self.role_results = False  # 占い結果をリセットする
                return ttf.divined(r[0], r[1])  # 占い結果を貼る

        elif role == u'MEDIUM':
            # 0日目は削除されるので新しいバージョンで試す場合は消す
            if self.today == 0 or self.today == 1:
                if not self.co:
                    return ttf.comingout(int(self.my[u'id']), "MEDIUM")  # 霊能COをする

            elif self.role_results and self.today != 1:  # 2日目以降でなおかつ結果を貼っていなかったら
                r = self.role_results
                self.role_results = False  # 占い結果をリセットする
                return ttf.inquested(r[0], r[1])  # 霊能結果を貼る

        # 役職発言以外の発言パターン
        # TALK_PATTERN = ["VOTE", "ESTIMATE WOLF", "ESTIMATE MEDIUM", "ESTIMATE SEER", "RESET_VOTE"]
        while True:
            talk_set = TALK_PATTERN[self.count]
            if talk_set == "VOTE":
                self.vote_target = create_vote(self.player_table, self.alive_player, self.today, self.my)
                self.count += 1
                return ttf.vote(self.vote_target)

            elif talk_set == "ESTIMATE WOLF":
                if self.wolf_estimate_count == 0:  # 初期定義
                    # リスト内の情報を全て入手する
                    self.wolf_estimate_list = create_wolf_table(self.player_table, self.alive_player, self.today, self.my)

                if len(self.wolf_estimate_list) == 0 or self.wolf_estimate_count == len(self.wolf_estimate_list):
                    self.count += 1
                    continue

                if role == u"SEER":  # 占い時にグレー推理すると怒られるのでパス
                    self.count += 1
                    continue

                target = self.wolf_estimate_list[self.wolf_estimate_count]  # 誘導先を入手する
                self.wolf_estimate_count += 1  # 疑う狼先を変える
                return ttf.estimate(int(target), u"WEREWOLF")

            elif talk_set == "ESTIMATE MEDIUM":  # 霊1ならそのプレイヤーを確霊宣言
                self.count += 1
                medium = role_player_id(self.player_table, u"MEDIUM")
                if len(medium) == 1:
                    return ttf.estimate(int(medium[0]), u"MEDIUM")

            elif talk_set == "ESTIMATE SEER":  # 占1ならそのプレイヤーを確占宣言

                if role == u"SEER":  # なんか怒られそうだからパス
                    self.count += 1
                    continue

                self.count += 1
                seer = role_player_id(self.player_table, u"SEER")
                if len(seer) == 1:
                    return ttf.estimate(int(seer[0]), u"SEER")

            elif talk_set == "RESET_VOTE":  # 投票先が変わったら宣言する
                new_vote_target = create_vote(self.player_table, self.alive_player, self.today, self.my)
                if self.vote_target != new_vote_target:
                    self.vote_target = new_vote_target
                    return ttf.vote(self.vote_target)

            return ttf.skip()

        return ttf.skip()  # バグを防ぐ為の処理

    def whisper(self, talk_history, whisper_history):
        # 噛み先を通知する予定
        self.player_table = detection(self.player_table ,talk_history)  # 発言の取得
        t = infer(self.player_table, self.alive_player, self.today, self.my)  # 色々な推理を一纏めにして行う
        inf = make_infer_table(t, self.alive_player, self.today)  # 推理テーブルを作成する
        target = int(kill_target(self.player_table, self.alive_player, self.my))
        return twf.attack(target)

    def dayFinish(self, talk_history, whisper_history):
        self.player_table = detection(self.player_table, talk_history)  # 発言の取得
        pass

    def vote(self, talk_history, whisper_history):
        # 投票先の計算
        if self.vote_target == 0:  # 定義されてないなら適当に入れとく
            self.vote_target = 1

        return self.vote_target

    def attack(self):  # 襲撃先を決める
        t = infer(self.player_table, self.alive_player, self.today, self.my)  # 色々な推理を一纏めにして行う
        inf = make_infer_table(t, self.alive_player, self.today)  # 推理テーブルを作成する
        target = int(kill_target(self.player_table, self.alive_player, self.my))
        return target  # 霊能→占い→狼投票者→村目

    def divine(self):  # 占い先を決める
        t = infer(self.player_table, self.alive_player, self.today, self.my)  # 色々な推理を一纏めにして行う
        inf = make_infer_table(t, self.alive_player, self.today)  # 推理テーブルを作成する
        target = int(seer_target(self.player_table, self.alive_player, inf, self.my[u'id']))
        return target  # 狼っぽいグレーを占う事にした（推理テーブル上位のプレイヤーでなおかつ村人）

    def guard(self):  # 護衛先を決める
        t = infer(self.player_table, self.alive_player, self.today, self.my)  # 色々な推理を一纏めにして行う
        inf = make_infer_table(t, self.alive_player, self.today)  # 推理テーブルを作成する
        target = int(guard_target(self.player_table, self.alive_player))
        return target

    def finish(self, game_info):
        pass
